/**
   This is a prism with side length 1, centered at the origin,
   and with its centeral axis on the y-axis.
   The total height is 2*(h + h2).
*/

public class PrismModel extends Model
{
   /**
      A prism with a regular tetrahedron at the top and bottom.
   */
   public PrismModel(double h)
   {
      this(h, (float)(Math.sqrt(2)/Math.sqrt(3)) );
   }


   /**
      A prism with a tetrahedron at the top and bottom, where
      the face-edge-face angle of the tetrahedron is theta.
      theta is in radians!
      (If theta = arctan(2*sqrt(2)), then the tetrathedron is regular.)
   */
   public PrismModel(double h, double theta)
   {
      this(h, (float)(Math.tan(theta)/(2*Math.sqrt(3))) );
   }


   /**
      A prism with a tetrahedron of height h2 at the top and bottom.
   */
   private PrismModel(double h, float h2)
   {
      super();

      // Create the prism's geometry.
      double sqrt3 = Math.sqrt(3.0);
      double sqrt2 = Math.sqrt(2.0);
      Vertex[] v = new Vertex[8];

      v[0] = new Vertex( 1.0/sqrt3, -h,  0);    // three vertices around the bottom
      v[1] = new Vertex(-0.5/sqrt3, -h,  0.5);
      v[2] = new Vertex(-0.5/sqrt3, -h, -0.5);

      v[3] = new Vertex( 1.0/sqrt3,  h,  0);    // three vertices around the top
      v[4] = new Vertex(-0.5/sqrt3,  h,  0.5);
      v[5] = new Vertex(-0.5/sqrt3,  h, -0.5);

      v[6] = new Vertex( 0, -h-h2, 0);  // vertex at the bottom
      v[7] = new Vertex( 0,  h+h2, 0);  // vertex at the top
/*
      // Regular tetrahedron with side length 1.
      v[0] = new Vertex( 1.0/sqrt3, -h,  0);    // three vertices around the bottom
      v[1] = new Vertex(-0.5/sqrt3, -h,  0.5);
      v[2] = new Vertex(-0.5/sqrt3, -h, -0.5);

      v[3] = new Vertex( 1.0/sqrt3,  h,  0);    // three vertices around the top
      v[4] = new Vertex(-0.5/sqrt3,  h,  0.5);
      v[5] = new Vertex(-0.5/sqrt3,  h, -0.5);

      v[6] = new Vertex( 0, -h-sqrt2/sqrt3, 0);  // vertex at the bottom
      v[7] = new Vertex( 0,  h+sqrt2/sqrt3, 0);  // vertex at the top


      // Regular tetrahedron with side length sqrt(3).
      v[0] = new Vertex( 1.0, -h,  0);         // three vertices around the bottom
      v[1] = new Vertex(-0.5, -h,  0.5*sqrt3);
      v[2] = new Vertex(-0.5, -h, -0.5*sqrt3);

      v[3] = new Vertex( 1.0,  h,  0);         // three vertices around the top
      v[4] = new Vertex(-0.5,  h,  0.5*sqrt3);
      v[5] = new Vertex(-0.5,  h, -0.5*sqrt3);

      v[6] = new Vertex( 0, -h-sqrt2, 0);  // vertex at the bottom
      v[7] = new Vertex( 0,  h+sqrt2, 0);  // vertex at the top


      // Regular tetrahedron with side length 2.
      v[0] = new Vertex( 2, -h,  0);      // three vertices around the bottom face
      v[1] = new Vertex(-1, -h,  sqrt3);
      v[2] = new Vertex(-1, -h, -sqrt3);

      v[3] = new Vertex( 2,  h,  0);      // three vertices around the top face
      v[4] = new Vertex(-1,  h,  sqrt3);
      v[5] = new Vertex(-1,  h, -sqrt3);

      v[6] = new Vertex( 0, -h-2*sqrt2, 0); // vertex at the very bottom
      v[7] = new Vertex( 0,  h+2*sqrt2, 0); // vertex at the very top
*/

      // Create 12 triangles, 2 for each face of the prism,
      // 3 triangles at the top, and 3 triangles at the bottom
      Triangle[] t = new Triangle[12];
      Vector n; // used for the normal vectors
      int i = 0;
      // 3 bottom faces
      n = (v[0].minus(v[6])).crossProduct(v[1].minus(v[6]));
      n = n.normalize();
      v[6].n = v[0].n = v[1].n = n;
      t[i++] = new Triangle(v[6], v[0], v[1]);
      n = (v[1].minus(v[6])).crossProduct(v[2].minus(v[6]));
      n = n.normalize();
      v[6].n = v[1].n = v[2].n = n;
      t[i++] = new Triangle(v[6], v[1], v[2]);
      n = (v[2].minus(v[6])).crossProduct(v[0].minus(v[6]));
      n = n.normalize();
      v[6].n = v[2].n = v[0].n = n;
      t[i++] = new Triangle(v[6], v[2], v[0]);
      // vertical face 1
      n = (v[1].minus(v[0])).crossProduct(v[3].minus(v[0]));
      n = n.normalize();
      v[0].n = v[1].n = v[3].n = v[4].n = n;
      t[i++] = new Triangle(v[0], v[1], v[4]);
      t[i++] = new Triangle(v[4], v[3], v[0]);
      // vertical face 2
      n = (v[2].minus(v[1])).crossProduct(v[4].minus(v[1]));
      n = n.normalize();
      v[1].n = v[2].n = v[4].n = v[5].n = n;
      t[i++] = new Triangle(v[1], v[2], v[5]);
      t[i++] = new Triangle(v[5], v[4], v[1]);
      // vertical face 3
      n = (v[0].minus(v[2])).crossProduct(v[5].minus(v[2]));
      n = n.normalize();
      v[2].n = v[0].n = v[5].n = v[3].n = n;
      t[i++] = new Triangle(v[2], v[0], v[3]);
      t[i++] = new Triangle(v[3], v[5], v[2]);
      // 3 top faces
      n = (v[3].minus(v[7])).crossProduct(v[4].minus(v[7]));
      n = n.normalize();
      v[7].n = v[3].n = v[4].n = n;
      t[i++] = new Triangle(v[7], v[3], v[4]);
      n = (v[4].minus(v[7])).crossProduct(v[5].minus(v[7]));
      n = n.normalize();
      v[7].n = v[4].n = v[5].n = n;
      t[i++] = new Triangle(v[7], v[4], v[5]);
      n = (v[5].minus(v[7])).crossProduct(v[3].minus(v[7]));
      n = n.normalize();
      v[7].n = v[5].n = v[3].n = n;
      t[i++] = new Triangle(v[7], v[5], v[3]);


      // Add the new triangles to the model.
      for (i = 0; i < t.length; i++)
      {
         triangleList.add( t[i] );
      }
   }
}