/**
   A LineLoop is like a LineStrip, but there is one
   additional line segment connecting the last given
   vertex in the first vertex. So a LineLoop represents
   a closed curve. A LineLoop with n vertices is a curve
   with n line segments.

   A LineLoop knows how to assemble itself into a list
   of LineSegment objects, so we can implementation the
   Primitive interface.
*/
import java.util.LinkedList;
import java.util.List;

public class LineLoop extends Primitive
{
   public LineLoop()
   {
      super();
   }


   /**
      Construct a LineLoop from an array of vertices.
   */
   public LineLoop(Vertex[] vArray)
   {
      super(vArray);
   }


   /**
      Construct a LineLoop from a List of vertices.
   */
   public LineLoop(List<Vertex> vList)
   {
      super(vList);
   }


   /**
      Create a LineLoop object whose vertices are copies of the vertices from the given LineLoop object.
   */
   public LineLoop(LineLoop loop) // a "copy constructor"
   {
      super();

      for (Vertex v : loop.vertexList)
      {
         this.vertexList.add( new Vertex(v) );  // make copies of each Vertex!
      }
   }


   /**
      We know how to assemble a line loop into a
      list of LineSegment objects.

      If there are n vertices in the line loop,
      then we need to create n LineSegment objects.
   */
   public List<LineSegment> assemble()
   {
      List<LineSegment> result = new LinkedList<LineSegment>();

      for (int i = 0; i < this.vertexList.size() - 1; i++)
      {
         LineSegment ls = new LineSegment(this.vertexList.get(i+0),
                                          this.vertexList.get(i+1));
         result.add( ls );
      }

      // close the line strip into a line loop
      result.add( new LineSegment(this.vertexList.get(this.vertexList.size() - 1),
                                  this.vertexList.get(0)) );

      return result;
   }


   /**
      Make a deep copy of this primitive.
   */
   public Primitive makeCopy()
   {
      return new LineLoop(this);
   }


   /**
      For debugging.
   */
   public String toString()
   {
      String result = "Line Loop:\n";
      for (Vertex v : this.vertexList)
      {
         result += v.toString() + "\n";
      }
      return result;
   }


   public String toString(int vertexFields)
   {
      String result = "Line Loop:\n";
      for (Vertex v : this.vertexList)
      {
         result += v.toString(vertexFields);
      }
      return result;
   }
}