/**
   A LineStrip is a collection of vertices that
   represent consecutively connected line segments.
   A LineStrip represents a curve. If the LineStrip
   contains n vertices, then the curve is made up of
   n-1 line segments.

   A LineStrip knows how to assemble itself into a list
   of LineSegment objects, so we can implementation the
   Primitive interface.
*/
import java.util.LinkedList;
import java.util.List;

public class LineStrip extends Primitive
{
   public LineStrip()
   {
      super();
   }


   /**
      Construct a LineStrip from an array of vertices.
   */
   public LineStrip(Vertex[] vArray)
   {
      super(vArray);
   }


   /**
      Construct a LineStrip from a List of vertices.
   */
   public LineStrip(List<Vertex> vList)
   {
      super(vList);
   }


   /**
      Create a LineStrip object whose vertices are copies of the vertices from the given LineStrip object.
   */
   public LineStrip(LineStrip ls) // a "copy constructor"
   {
      super();

      for (Vertex v : ls.vertexList)
      {
         this.vertexList.add( new Vertex(v) );  // make copies of each Vertex!
      }
   }


   /**
      We know how to assemble a line strip into a
      list of LineSegment objects.

      If there are n vertices in the line strip,
      then we need to create n-1 LineSegment objects.
   */
   public List<LineSegment> assemble()
   {
      List<LineSegment> result = new LinkedList<LineSegment>();

      for (int i = 0; i < this.vertexList.size() - 1; i++)
      {
         LineSegment ls = new LineSegment(this.vertexList.get(i+0),
                                          this.vertexList.get(i+1));
         result.add( ls );
      }

      return result;
   }


   /**
      Make a deep copy of this primitive.
   */
   public Primitive makeCopy()
   {
      return new LineStrip(this);
   }


   /**
      For debugging.
   */
   public String toString()
   {
      String result = "Line Strip:\n";
      for (Vertex v : this.vertexList)
      {
         result += v.toString() + "\n";
      }
      return result;
   }


   public String toString(int vertexFields)
   {
      String result = "Line Strip:\n";
      for (Vertex v : this.vertexList)
      {
         result += v.toString(vertexFields);
      }
      return result;
   }
}