/**
   A Model data structure represents a distinct geometric object (in a Scene). A Model data structure is mainly a list of Primitive objects, where each primitive object represents a collection of line segments. The line segments represent the geometric object as a "wire-frame", that is, the geometric object is drawn as a collection of "edges". This is a fairly simplistic way of doing 3D graphics and we will improve this in a later renderer.

   See
      http://en.wikipedia.org/wiki/Wire-frame_model
   or
      https://www.google.com/search?q=graphics+wireframe&tbm=isch


   In this renderer, a Model also contains a model matrix, which determines the model's location in world coordinates. The model matrix helps us solve the "instancing problem" when we want to have multiple instances of a model in a scene.

   In this renderer, a Model also contains a List of sub Models. This list of sub models helps us define what are called hierarchical, or nested, models. These are models that are made up of distinct parts and the distinct parts need to both move individually and also move together as a unit.

   As a simple, and common, example of a "nested model", think of a robot arm made up of a upper-arm, lower-arm, wrist, and five fingers. When you move the arm at its shoulder, all the arm's parts should move together. But when you move the arm at its elbow, only the lower-arm, wrist and fingers should move toghether. When you move the arm at its wrist, the wrist and fingers should move together. And each finger should be able to move by itself.
*/
import java.util.LinkedList;
import java.util.List;
import java.awt.Color;

public class Model
{
   public Matrix modelMatrix = new Matrix(1.0); // identity matrix

   public List<Primitive> primitiveList = new LinkedList<Primitive>();

   public List<Model> nestedModels = new LinkedList<Model>();

   public boolean doBackFaceCulling = true;
   public boolean  frontFacingIsCCW = true;
   public boolean facesHaveTwoSides = false;
   public Color frontFaceColor      = Color.black;
   public Color backFaceColor       = Color.lightGray;


   public Model()
   {
   }


   /**
      Duplicate a Model object.
   */
   public Model(Model model)
   {
      // Copy everything except the list of geometric primitives
      // and the list of sub-models.
      this.modelMatrix       = model.modelMatrix;
      this.doBackFaceCulling = model.doBackFaceCulling;
      this.frontFacingIsCCW  = model.frontFacingIsCCW;
      this.facesHaveTwoSides = model.facesHaveTwoSides;
      this.frontFaceColor    = model.frontFaceColor;
      this.backFaceColor     = model.backFaceColor;
   }//duplicating constructor


   /**
      Add a Primitive to this model's list of geometric primitives.
   */
   public void addPrimitive(Primitive p)
   {
      this.primitiveList.add(p);
   }


   /**
      Set all the primitives in this model to the same color.
   */
   public void setColor(Color c)
   {
      for (Primitive p : this.primitiveList)
      {
         p.setColor(c);
      }
   }


   /**
      Set all the primitives in this model to random colors.
   */
   public void setRandomColors()
   {
      for (Primitive p : this.primitiveList)
      {
         p.setColorRandom();
      }
   }


   public void model2Identity()
   {
      this.modelMatrix = new Matrix(1.0);
   }

   public void modelMult(Matrix m)
   {
      this.modelMatrix = this.modelMatrix.times(m);
   }

   public void modelRotate(double theta, double x, double y, double z)
   {
      this.modelMult(new Matrix(theta, x, y, z));
   }

   public void modelScale(double x, double y, double z)
   {
      this.modelMult(new Matrix(x, y, z));
   }

   public void modelTranslate(double x, double y, double z)
   {
      this.modelMult(new Matrix(new Vector(x, y, z)));
   }


   public void addSubModel(Model m)
   {
      this.nestedModels.add(m);
   }

   public int indexOfSubModel(Model m)
   {
      return this.nestedModels.indexOf(m);
   }

   public void setSubModel(int index, Model m)
   {
      this.nestedModels.set(index, m);
   }

   public Model getSubModel(int index)
   {
      return nestedModels.get(index);
   }


   /**
      For debugging.
   */
   public String toString()
   {
      String result = "";
      result += "This Model has " + primitiveList.size() + " primitives\n";
      //result = "Printing out this Model's " + primitiveList.size() + " Primitives:\n";
      for (Primitive p : this.primitiveList)
      {
         result += p.toString();
      }
      result += "This Model has " + nestedModels.size() + " nested models\n";
      for (Model m : this.nestedModels)
      {
         result += m.toString();
      }
      //result += "Done printing out Model\n";
      return result;
   }


   /**
      Used for debugging. What this method returns
      depends on the value of the parameter vertexFields.

      The value of vertexFields is treated as a bit field.
      The bits have the following meanings.
        1 => print vertex coordinates
        2 => print r g b values
   */
   public String toString(int vertexFields)
   {
      String result = "";
      result += "This Model has " + primitiveList.size() + " primitives\n";
      //result = "Printing out this Model's " + primitiveList.size() + " Primitives:\n";
      for (Primitive p : this.primitiveList)
      {
         result += p.toString(vertexFields);
      }
      result += "This Model has " + nestedModels.size() + " nested models\n";
      for (Model m : this.nestedModels)
      {
         result += m.toString(vertexFields);
      }
      //result += "Done printing out Model\n";
      return result;
   }
}