/**
   A Primitive is something that we can build geometric
   shapes out of (a "graphics primitive").

   The most "primitive" primitive is a LineSegment.

   In the "primitive assembly" stage of the rendering
   pipeline, each "higher order" Primitive needs to be
   able to "assemble" itself into a list of "lower order"
   LineSegment objects (for the clipping and rasterization
   stages of the rendering pipeline). Since this abstractly
   defined primitive does not yet know how to assemble
   itself, the assemble() method is an abstract method.
*/
import java.util.ArrayList;
import java.util.List;
import java.awt.Color;

public abstract class Primitive
{
   // A Primitive object is made up of vertices.
   protected List<Vertex> vertexList = null;


   protected Primitive()
   {
      this.vertexList = new ArrayList<Vertex>();
   }


   /**
      Construct a Primitive from an array of vertices.
   */
   protected Primitive(Vertex[] vArray)
   {
      this();

      for (Vertex v : vArray)
      {
         this.vertexList.add( new Vertex(v) );  // make copies of each Vertex!
      }
   }


   /**
      Construct a Primitive from a List of vertices.
   */
   protected Primitive(List<Vertex> vList)
   {
      this();

      for (Vertex v : vList)
      {
         this.vertexList.add( new Vertex(v) );  // make copies of each Vertex!
      }
   }


   /**
      Add a vertex at the end of the vertex list.
   */
   public void addVertex(Vertex v)
   {
      this.vertexList.add( new Vertex(v) );  // make a copy of the Vertex!
   }


   /**
      Give this primitive a uniform color.
   */
   public void setColor(Color c)
   {
      for (Vertex v : this.vertexList)
      {
         v.setColor(c);
      }
   }

   /**
      Give this primitive a uniform, but randomly chosen, color.
   */
   public void setColorRandom()
   {
      vertexList.get(0).setColorRandom();
      Color c = vertexList.get(0).getColor();
      for (Vertex v : this.vertexList)
      {
         v.setColor(c);
      }
   }


   /**
      Make a deep copy of this primitive.
   */
   public abstract Primitive makeCopy();


   /**
      Assemble this primitive into a list of LineSegment primitives.

      This is needed in the primitive-assembly stage of the rendering pipeline.
   */
   public abstract List<LineSegment> assemble();


   /**
      For debugging.
   */
   public abstract String toString(int vertexFields);
}