/**
   Cull from the list of primitives any Face that is "back facing".

   For a good explanation of orientation, forward and backward facing polygons,
   and back face culling, see Chapter 9 of
      "Computer Graphics Through OpenGL: From Theory to Experiments "
       by Sumanta Guha, Chapman and Hall/CRC, 2010.
  That chapter is available at
       http://www.sumantaguha.com/files/materials/ch9.pdf
*/

public class P3_BackFaceCulling extends PipelineStage
{
   /**
      This constructor initializes this object's
      reference to the SceneRender object that
      contains it. This gives this stage access
      to addditional information that might be
      needed to process the Models passed to the
      processModel() method.
   */
   public P3_BackFaceCulling(SceneRender sceneRender)
   {
      super(sceneRender);
   }


   /**
      Walk the Model's list of Primitive objects.
      Determine if a Face is forward or
      backward facing. Delete any Face
      that is backward facing.
   */
   public Model processModel(Model model)
   {
      if (model.doBackFaceCulling)
      {
         java.util.Iterator<Primitive> itr = model.primitiveList.iterator();

         while ( itr.hasNext() )
         {
            Primitive face = itr.next();
            if (face instanceof Face)
            {
               if ( ! faceIsFrontFacing(model.frontFacingIsCCW, (Face)face) )
               {
                  itr.remove(); // unlink this Face
               }
            }
         }
      }
      else if (model.facesHaveTwoSides)
      {
         for (Primitive p : model.primitiveList)
         {
            if (p instanceof Face)
            {
               if ( faceIsFrontFacing(model.frontFacingIsCCW, (Face)p) )
               {
                  p.setColor( model.frontFaceColor );
               }
               else
               {
                  p.setColor( model.backFaceColor );
               }
            }
         }
      }
      return model;
   }


   private boolean faceIsFrontFacing(boolean frontFacingIsCCW, Face face)
   {
      boolean result = faceIsCCW(face);
      if ( ! frontFacingIsCCW )
         result = ! result;
      return result;
   }


   /**
      In view coordinates, this version is accurate when using
      an orthographic projection but it is only approximately
      correct when using a perspective projection.

      Also, this version can be used in Normalized Device Coordinates
      (after the projection stage).
   */
   private boolean faceIsCCW_(Face face)
   {
      // We only need to know the z component of the cross product
      // of the two "edge vectors", v1-v0 and v2-v0. This works
      // because we are in view coordinates, with the camera at
      // the origin looking down the negative z-axis.
      double x0 = face.vertexList.get(0).x;
      double x1 = face.vertexList.get(1).x;
      double x2 = face.vertexList.get(2).x;
      double y0 = face.vertexList.get(0).y;
      double y1 = face.vertexList.get(1).y;
      double y2 = face.vertexList.get(2).y;
      double zValueOfCrossProduct = (x1-x0)*(y2-y0) - (y1-y0)*(x2-x0);

      //System.err.printf("zValueOfCrossProduct = % .6f\n", zValueOfCrossProduct);

      return (zValueOfCrossProduct >= 0);
   }


   /**
      In view coordinates, this version is accurate when
      using a perspective projection.
   */
   private boolean faceIsCCW(Face face)
   {
      double x0 = face.vertexList.get(0).x;
      double x1 = face.vertexList.get(1).x;
      double x2 = face.vertexList.get(2).x;
      double y0 = face.vertexList.get(0).y;
      double y1 = face.vertexList.get(1).y;
      double y2 = face.vertexList.get(2).y;
      double z0 = face.vertexList.get(0).z;
      double z1 = face.vertexList.get(1).z;
      double z2 = face.vertexList.get(2).z;
      double determinant = x0*(y1*z2-y2*z1)-x1*(y0*z2-y2*z0)+x2*(y0*z1-y1*z0);

      //System.err.printf("determinant = % .6f\n", determinant);

      return (determinant < 0);
   }
}