/**
   Project primitives from view coordinates to clip coordinates.

   For each Model in a Scene, walk the Model's list of Primitive objects.
   For each Vertex object in each Primitive object, use the Scene's
   "projection matrix" to project the Vertex object into the front plane
   of the Scene's view volume.

   After projecting a vertex into the front plane of the view volume,
   "normalize" the coordinates of the vertex so that the front *face* of
   the Scene's view volume is represented by points with x and y coordinates
   between -1 and 1. (These new coordinates are called "clip coordinates" or
   "normalized device coordinates".)
*/

public class P4_View2Clip extends PipelineStage
{
   /**
      This constructor initializes this object's
      reference to the SceneRender object that
      contains it. This gives this stage access
      to addditional information that might be
      needed to process the Models passed to the
      processModel() method.
   */
   public P4_View2Clip(SceneRender sceneRender)
   {
      super(sceneRender);
   }


   /**
      For each primitive in the model, project each vertex
      in the primitive to the front plane of the view volume
      and then transform each vertex from view coordinates
      to clip coordinates.
   */
   public Model processModel(Model model)
   {
      Camera camera = this.sceneRender.scene.camera;

      double left   = camera.left;   // These numbers define the camera's "view volume".
      double right  = camera.right;  // (In future renderers, these numbers will be
      double bottom = camera.bottom; //  encoded into a "projection matrix".)
      double top    = camera.top;
      double near   = camera.near;
      double far    = camera.far;

      // Project each geometric primitive in the model's primitive list.
      for (Primitive p : model.primitiveList)
      {
         // For each vertex in the primitive, compute its projected (x,y) coordinates.
         for (Vertex v : p.vertexList)
         {
            double v_xp = -near * (v.x / v.z);  // xp = near * (xv / -zv)
            double v_yp = -near * (v.y / v.z);  // yp = near * (yv / -zv)

            if ( ! camera.perspective ) // do parallel projection instead
            {
               v_xp = v.x;
               v_yp = v.y;
            }

            // Normalize the projected coordinates to "clip coordinates".
            double v_xc = ((2 * v_xp) - right - left)/(right - left);
            double v_yc = ((2 * v_yp) - top - bottom)/(top - bottom);

            // Mutate the Vertex object so that it contain the (homogeneous) clip coordinates.
            v.x = v_xc;
            v.y = v_yc;
            v.z = 0.0;
            v.w = 1.0;
         }
      }
      return model;
   }
}