/**
   This class implements the "rendering pipeline".

   This class contains a Vector of PipelineStage
   objects and those objects make up the renderer.

   Notice that a stage for this rendering pipeline can
   be implemented by any client program, and then that
   (client defined) stage can be inserted into the
   Vector of PipelineStage objects maintained by this
   SceneRender object. In this way, we have a client
   configurable, and programmable, pipeline (as with
   the modern OpenGL 4 pipeline).
*/
import java.util.Vector;

public class SceneRender
{
   private Vector<PipelineStage> pipeline;
   public Scene scene;
   public FrameBuffer fb;

   /**
      This constructor initializes the default rendering pipeline.

      A client program can replace, delete, or insert other stages
      into this pipeline.

      At the end of the default rendering process, the Framebuffer's
      viewport holds the rendered image of the scene represented
      by the Scene object.
   */
   public SceneRender()
   {
      this.pipeline = new Vector<PipelineStage>();

      // Build the default pipeline.

      // Duplicate a model.
      pipeline.add( new P0_CopyModel(this) );
      // Apply the model transformation.
      pipeline.add( new P1_Model2World(this) );
      // Apply the view transformation.
      pipeline.add( new P2_World2View(this) ); // needs the view transformation matrix
      // Remove all the faces that are "back facing".
      pipeline.add( new P3_BackFaceCulling(this) );
      // Apply the projection transformation.
      pipeline.add( new P4_View2Clip(this) );  // needs the projection transformation
      // Assemble higher order primitives into LineSegments.
      pipeline.add( new P5_PrimitiveAssembly(this) );
      // Clip line segments to the standard volume.
      pipeline.add( new P6_Clipping(this) );
      // Transform normalized device coordinates to viewport coordinates.
      pipeline.add( new P7_NDC2VP(this) ); // needs the viewport dimensions
      // Rasterize each line segment into pixels.
      pipeline.add( new P8_Rasterize(this) ); // needs the pixel buffer
   }


   /**
      This method renders the Scene object by passing each
      of its Model objects through the rendering pipeline.
   */
   public Scene renderScene(Scene scene, FrameBuffer fb)
   {
      this.scene = scene;
      this.fb = fb;

      Scene renderedScene = new Scene(scene); // copy everything but the geometry

      int modelNumber = 1;  // useful for debugging
      for (Model model : scene.modelList)
      {
         // Send this model down the pipeline.
         int stageNumber = 0;  // useful for debugging
         for (PipelineStage ps : pipeline)
         {
            model = ps.processModel(model);

            // debugging output (depends on environment variables)
            printVertices(model, stageNumber, ps.getClass().getName());

            stageNumber++;
         }
         // Try saving the FrameBuffer to a file at this point.
       //fb.dumpFB2File( "fb_after_model_" + modelNumber + ".ppm");

         renderedScene.addModel(model);
         modelNumber++;
      }
      return renderedScene;
   }//renderScene()


   /**
      This method replaces the pipeline stage at the
      given index with the new PipelineStage object.
   */
   public void replacePipelineStage(int index, PipelineStage ps)
   {
      pipeline.set(index, ps);
   }


   /**
      This method inserts the new pipeline stage at the
      given index. The current PipelineStage object at
      that index, and all subsequent pipeline stage
      objects, shift up in the pipeline.
   */
   public void insertPipelineStage(int index, PipelineStage ps)
   {
      pipeline.add(index, ps);
   }


   /**
      This method deletes the pipeline stage at the
      given index. All subsequent pipeline stage objects
      shift down in the pipeline.
   */
   public PipelineStage removePipelineStage(int index)
   {
      return pipeline.remove(index);
   }


   /**
      This method will create a string containing the names
      of the pipeline stages. This is useful for debugging.
   */
   public String toString()
   {
      String result = "";
      for (PipelineStage ps : pipeline)
      {
         result += ps.getClass().getName() + "\n";
      }
      return result;
   }


   /**
      Used for debugging.
   */
   private static void printVertices(Model model, int stageNo, String stageName)
   {
      int stageFlag = 1;
      for (int i = 0; i < stageNo; i++)
         stageFlag = stageFlag << 1;  // shift the 1 bit into the correct position

      int debugStages = 0;
      String debugString = null;
      if ( (debugString = System.getenv("PipeLine0")) != null ) debugStages += 1;
      if ( (debugString = System.getenv("PipeLine1")) != null ) debugStages += 2;
      if ( (debugString = System.getenv("PipeLine2")) != null ) debugStages += 4;
      if ( (debugString = System.getenv("PipeLine3")) != null ) debugStages += 8;
      if ( (debugString = System.getenv("PipeLine4")) != null ) debugStages += 16;
      if ( (debugString = System.getenv("PipeLine5")) != null ) debugStages += 32;
      if ( (debugString = System.getenv("PipeLine6")) != null ) debugStages += 64;
      if ( (debugString = System.getenv("PipeLine7")) != null ) debugStages += 128;
      if ( (debugString = System.getenv("PipeLine8")) != null ) debugStages += 256;

      int vertexFields = 1;
      String printFields = null;
      if ( (printFields = System.getenv("PrintFields")) != null )
      {
         vertexFields = Integer.parseInt(printFields);
      }

      if ((stageFlag & debugStages) != 0)
      {
         System.err.printf("=============== begin Stage %d, %s =============\n", stageNo, stageName);
         System.err.print( model.toString(vertexFields) );
         System.err.printf("================= end Stage %d, %s =============\n", stageNo, stageName);
      }
   }//printVertices()
}