/**
   This is a right, triangular prism with length 2h, side length 1, centered
   at the origin, and with its central axis on the y-axis.
   http://en.wikipedia.org/wiki/Prism_(geometry)

   Attached to each of the two triangular ends of the prism is a tetrahedron
   of height h2. So the total height is 2*(h + h2).
*/

public class PrismModel extends Model
{
   /**
      A prism with a regular tetrahedron at the top and bottom.
   */
   public PrismModel(int n, double h)
   {
      this(n, h, (float)(Math.sqrt(2)/Math.sqrt(3)) );
   }


   /**
      A prism with a tetrahedron at the top and bottom, where
      the face-edge-face angle of the tetrahedron is theta.
      theta is in radians!
      (If theta = arctan(2*sqrt(2)), then the tetrahedron is regular.)
   */
   public PrismModel(int n, double h, double theta)
   {
      this(n, h, (float)(Math.tan(theta)/(2*Math.sqrt(3))) );
   }


   /**
      A prism of length h with a tetrahedron of height h2 at the top and bottom.
      NOTE: h2 is a float so that we can distinguish this constructor from the previous one.
   */
   private PrismModel(int n, double h, float h2)
   {
      super();

      // Create the prism's geometry.
      double sqrt3 = Math.sqrt(3.0);
      double sqrt2 = Math.sqrt(2.0);
      Vertex[] v = new Vertex[8];

      v[0] = new Vertex( 1.0/sqrt3,  h,  0);    // three vertices around the top
      v[1] = new Vertex(-0.5/sqrt3,  h,  0.5);
      v[2] = new Vertex(-0.5/sqrt3,  h, -0.5);

      v[3] = new Vertex( 1.0/sqrt3, -h,  0);    // three vertices around the bottom
      v[4] = new Vertex(-0.5/sqrt3, -h,  0.5);
      v[5] = new Vertex(-0.5/sqrt3, -h, -0.5);

      v[6] = new Vertex( 0,  h+h2, 0);  // vertex at the top
      v[7] = new Vertex( 0, -h-h2, 0);  // vertex at the bottom

      // Create two line fans and two line loops
      Primitive  fan1 = new LineFan();  // Line fan from the top
      Primitive  fan2 = new LineFan();  // Line fan from the bottom
      Primitive loop1 = new LineLoop(); // Loop around the top edge
      Primitive loop2 = new LineLoop(); // Loop around the bottom edge
      fan1.addVertex( v[6] );
      fan2.addVertex( v[7] );
      for (int j = 0; j < 3; j++)
      {
          fan1.addVertex( v[j] );
         loop1.addVertex( v[j] );

          fan2.addVertex( v[3+j] );
         loop2.addVertex( v[3+j] );
      }
      this.addPrimitive( fan1 );
      this.addPrimitive( fan2 );
      this.addPrimitive( loop1 );
      this.addPrimitive( loop2 );

      // Create the three vertical lines from the top to the bottom.
      for (int j = 0; j < 3; j++)
      {
         this.addPrimitive( new LineSegment(v[j], v[3+j]) );
      }

      if (n > 0)
      {
         // Create n lines of "latitude" around the prism.
         Vertex[][] v2 = new Vertex[n][3];

         for (int j = 0; j < n; j++)
         {
            double y = i * (h/n);
            v2[j][0] = new Vertex( 1.0/sqrt3,  y,  0);
            v2[j][1] = new Vertex(-0.5/sqrt3,  y,  0.5);
            v2[j][2] = new Vertex(-0.5/sqrt3,  y, -0.5);

            this.addPrimitive( new LineLoop(v2[j]) );
         }
      }
   }
}